@@ -24,7 +24,7 @@
 #include <errno.h>
 #include <string.h>
 #include <ctype.h>
-#include <fstream.h>
+#include <fstream>
 
 #include "Toc.h"
 #include "util.h"
@@ -49,6 +49,26 @@ Toc::Toc() : length_(0)
   catalogValid_ = 0;
 }
 
+// copy constructor
+Toc::Toc(const Toc &obj) : length_(0), cdtext_(obj.cdtext_)
+{
+  tocType_ = obj.tocType_;
+
+  if ((catalogValid_ = obj.catalogValid_))
+    memcpy(catalog_, obj.catalog_, 13);
+
+  nofTracks_ = 0;
+  tracks_ = lastTrack_ = NULL;
+
+  // copy all tracks
+  TrackIterator itr(&obj);
+  const Track *trun;
+
+  for (trun = itr.first(); trun != NULL; trun = itr.next())
+    append(trun);
+}
+
+
 Toc::~Toc()
 {
   TrackEntry *run = tracks_;
@@ -143,7 +163,7 @@ int Toc::write(const char *filename) const
   assert(filename != NULL);
   assert(*filename != 0);
 
-  ofstream out(filename);
+  std::ofstream out(filename);
 
   if (!out) {
     message(-2, "Cannot open file \"%s\" for writing: %s", filename,
@@ -216,7 +236,7 @@ const char *Toc::catalog() const
 }
 
 // writes contents in TOC file syntax
-void Toc::print(ostream &out) const
+void Toc::print(std::ostream &out) const
 {
   int i;
   TrackEntry *t;
@@ -228,7 +248,7 @@ void Toc::print(ostream &out) const
     for (i = 0; i < 13; i++) {
       out << (char)(catalog(i) + '0');
     }
-    out << "\"" << endl;
+    out << "\"" << std::endl;
   }
 
   cdtext_.print(0, out);
@@ -236,7 +256,7 @@ void Toc::print(ostream &out) const
   for (t = tracks_, i = 1; t != NULL; t = t->next, i++) {
     out << "\n// Track " << i << "\n";
     t->track->print(out);
-    out << endl;
+    out << std::endl;
   }
 }
 
@@ -297,11 +317,15 @@ int Toc::moveTrackMarker(int trackNr, int indexNr, long lba)
     return 2;
   }
 
-  if (indexNr <= 1 && act->track->type() != TrackData::AUDIO)
+  if (indexNr <= 1 && 
+      (act->track->type() != TrackData::AUDIO ||
+       act->track->subChannelType() != TrackData::SUBCHAN_NONE))
     return 6;
 
   if ((indexNr == 0 || (indexNr == 1 && act->track->start().lba() == 0)) &&
-      act->pred != NULL && act->pred->track->type() != TrackData::AUDIO)
+      act->pred != NULL && 
+      (act->pred->track->type() != TrackData::AUDIO ||
+       act->track->subChannelType() != TrackData::SUBCHAN_NONE))
     return 6;
   
 
@@ -433,10 +457,14 @@ int Toc::removeTrackMarker(int trackNr, int indexNr)
     return 2;
   }
 
-  if (act->track->type() != TrackData::AUDIO && indexNr <= 1)
+  if ((act->track->type() != TrackData::AUDIO ||
+       act->track->subChannelType() != TrackData::SUBCHAN_NONE) &&
+      indexNr <= 1)
     return 3;
 
-  if (act->pred != NULL && act->pred->track->type() != TrackData::AUDIO &&
+  if (act->pred != NULL && 
+      (act->pred->track->type() != TrackData::AUDIO ||
+       act->track->subChannelType() != TrackData::SUBCHAN_NONE) &&
       indexNr <= 1)
     return 3;
 
@@ -525,6 +553,9 @@ int Toc::addTrackMarker(long lba)
   if (act->track->type() != TrackData::AUDIO)
     return 5;
 
+  if (act->track->subChannelType() != TrackData::SUBCHAN_NONE)
+    return 5;
+
   if (lba <= act->start.lba())
     return 2;
   
@@ -538,7 +569,7 @@ int Toc::addTrackMarker(long lba)
   TrackDataList *dataList =
     act->track->removeToEnd(Msf(lba - act->absStart.lba()).samples());
 
-  Track *t = new Track(act->track->type());
+  Track *t = new Track(act->track->type(), act->track->subChannelType());
   t->appendTrackData(dataList);
   delete dataList;
 
@@ -579,11 +610,15 @@ int Toc::addPregap(long lba)
 
   if (act->track->type() != TrackData::AUDIO) 
     return 4;
+
+  if (act->track->subChannelType() != TrackData::SUBCHAN_NONE)
+    return 4;
   
   if (act->next == NULL)
     return 2; // no next track where we could add pre-gap
 
-  if (act->next->track->type() != TrackData::AUDIO) 
+  if (act->next->track->type() != act->track->type() ||
+      act->next->track->subChannelType() != act->track->subChannelType())
     return 4;
   
   if (lba <= act->start.lba())
@@ -634,7 +669,7 @@ void Toc::checkConsistency()
 // first LBA of new track, 'end' is filled with last LBA + 1 of new track.
 void Toc::appendTrack(const TrackDataList *list, long *start, long *end)
 {
-  Track t(TrackData::AUDIO);
+  Track t(TrackData::AUDIO, TrackData::SUBCHAN_NONE);
   const TrackData *run;
 
   for (run = list->first(); run != NULL; run = list->next())
@@ -645,8 +680,7 @@ void Toc::appendTrack(const TrackDataList *list, long *start, long *end)
   unsigned long len = list->length();
 
   if (len < minTime)
-    t.append(SubTrack(SubTrack::DATA,
-		      TrackData(TrackData::AUDIO, minTime - len)));
+    t.append(SubTrack(SubTrack::DATA, TrackData(minTime - len)));
 
   *start = length().lba();
 
@@ -673,6 +707,10 @@ int Toc::appendTrackData(const TrackDataList *list, long *start, long *end)
   if (lastTrack_->track->type() != TrackData::AUDIO)
     return 1;
 
+  if (lastTrack_->track->subChannelType() != TrackData::SUBCHAN_NONE)
+    return 1;
+
+
   *start = length().lba();
 
   for (run = list->first(); run != NULL; run = list->next())
@@ -702,6 +740,9 @@ int Toc::removeTrackData(unsigned long start, unsigned long end,
   if (tent->track->type() != TrackData::AUDIO)
     return 2;
 
+  if (tent->track->subChannelType() != TrackData::SUBCHAN_NONE)
+    return 2;
+
   if (tent != findTrack(end))
     return 1;
 
@@ -724,6 +765,10 @@ int Toc::insertTrackData(unsigned long pos, const TrackDataList *list)
 
   if (tent != NULL && tent->track->type() != TrackData::AUDIO)
     return 1;
+
+  if (tent != NULL && tent->track->subChannelType() != TrackData::SUBCHAN_NONE)
+    return 1;
+
   
   if (tent != NULL) {
     tent->track->insertTrackData(pos - tent->absStart.samples(), list);
@@ -1118,6 +1163,13 @@ const Track *TrackIterator::first(Msf &start, Msf &end)
   return next(start, end);
 }
 
+const Track *TrackIterator::first()
+{
+  Msf start, end;
+
+  return first(start, end);
+}
+
 const Track *TrackIterator::next(Msf &start, Msf &end)
 {
   Track *t;
@@ -1134,6 +1186,12 @@ const Track *TrackIterator::next(Msf &start, Msf &end)
   }
 }
 
+const Track *TrackIterator::next()
+{
+  Msf start, end;
+
+  return next(start, end);
+}
 
 // Class TocReader
 
@@ -1203,6 +1261,7 @@ void TocReader::closeData()
   }
 }
 
+#if 0
 long TocReader::readData(long lba, char *buf, long len)
 {
   long n;
@@ -1245,6 +1304,7 @@ long TocReader::readData(long lba, char *buf, long len)
  
   return nread;
 }
+#endif
 
 // seeks to specified sample (absolute position)
 // return: 0: OK
