@@ -21,7 +21,7 @@
 #ifndef __TRACKDATA_H__
 #define __TRACKDATA_H__
 
-#include <iostream.h>
+#include <iostream>
 #include "Sample.h"
 
 #define AUDIO_BLOCK_LEN 2352
@@ -32,12 +32,20 @@
 #define MODE2_FORM1_DATA_LEN 2048
 #define MODE2_FORM2_DATA_LEN 2324
 
+#define PQ_SUBCHANNEL_LEN 16
+#define PW_SUBCHANNEL_LEN 96
+#define MAX_SUBCHANNEL_LEN 96
+
+#define SAMPLE_LEN 4
+
 class TrackData {
 public:
   enum Mode { AUDIO, MODE0, MODE1, MODE2, MODE2_FORM1, MODE2_FORM2,
               MODE2_FORM_MIX, MODE1_RAW, MODE2_RAW };
 
-  enum Type { DATAFILE, ZERODATA, STDIN };
+  enum SubChannelMode { SUBCHAN_NONE, SUBCHAN_RW, SUBCHAN_RW_RAW };
+
+  enum Type { DATAFILE, ZERODATA, STDIN, FIFO };
   
   enum FileType { RAW, WAVE };
 
@@ -46,13 +54,18 @@ public:
 	    unsigned long length);
   TrackData(const char *filename, unsigned long start, unsigned long length);
 
+  // creates a zero audio entry
+  TrackData(unsigned long length);
+
   // creates a zero data entry with given mode
-  TrackData(Mode, unsigned long length);
+  TrackData(Mode, SubChannelMode, unsigned long length);
 
   // creates a file entry with given mode
-  TrackData(Mode, const char *filename, long offset,
+  TrackData(Mode, SubChannelMode, const char *filename, long offset,
 	    unsigned long length);
-  TrackData(Mode, const char *filename, unsigned long length);
+
+  // create a fifo entry with given mode
+  TrackData(Mode, SubChannelMode, const char *filename, unsigned long length);
 
   // copy constructor
   TrackData(const TrackData &);
@@ -61,6 +74,8 @@ public:
 
   Type type() const;
   Mode mode() const;
+  SubChannelMode subChannelMode() const;
+  int audioCutMode() const;
 
   const char *filename() const;
   unsigned long startPos() const;
@@ -76,7 +91,7 @@ public:
   void split(unsigned long, TrackData **part1, TrackData **part2);
   TrackData *merge(const TrackData *) const;
 
-  void print(ostream &) const;
+  void print(std::ostream &) const;
 
   static int checkAudioFile(const char *fn, unsigned long *length);
   static int waveLength(const char *filename, long offset, long *hdrlen,
@@ -86,12 +101,22 @@ public:
   static FileType audioFileType(const char *filename);
   static int dataFileLength(const char *fname, long offset,
 			    unsigned long *length);
-  static unsigned long dataBlockSize(Mode m);
-  static const char *mode2String(Mode m);
+
+  static unsigned long dataBlockSize(Mode, SubChannelMode);
+  static unsigned long subChannelSize(SubChannelMode);
+
+  static const char *mode2String(Mode);
+  static const char *subChannelMode2String(SubChannelMode);
 
 private:
+  int audioCutMode_; /* defines if audio cut mode is requested, if yes
+			all lengths are in sample units and sub-channel
+			data is not allowed
+		     */
   Type type_; // type of data (file, stdin, zero)
   Mode mode_; // data mode for recording (audio, mode0, mode1, ...)
+  SubChannelMode subChannelMode_; // sub-channel mode
+
   FileType fileType_; // only for audio mode data, type of file (raw, wave)
 
   char *filename_; // used for object type 'FILE'
@@ -110,7 +135,7 @@ private:
 
   void init(const char *filename, long offset, unsigned long start,
 	    unsigned long length);
-  void init(Mode, const char *filename, long offset,
+  void init(Mode, SubChannelMode, const char *filename, long offset,
 	    unsigned long length);
   
 
@@ -146,6 +171,12 @@ private:
   int readUnderRunMsgGiven_;
 };
 
+inline
+int TrackData::audioCutMode() const
+{
+  return audioCutMode_;
+}
+
 inline
 TrackData::Type TrackData::type() const
 {
@@ -158,6 +189,12 @@ TrackData::Mode TrackData::mode() const
   return mode_;
 }
 
+inline
+TrackData::SubChannelMode TrackData::subChannelMode() const
+{
+  return subChannelMode_;
+}
+
 inline
 const char *TrackData::filename() const
 {
